#include <iostream>
#include "Eigen/Dense"

/*
More at: http://eigen.tuxfamily.org/dox/index.html
*/

using namespace std;
using namespace Eigen;

int main()
{
    //An example 2x2 matrix od doubles and initialization
    MatrixXd m(2,2);
    m(0,0) = 3; m(1,0) = 2.5;
    m(0,1) = -1; m(1,1) = m(1,0) + m(0,1);
    std::cout << "Here is the matrix m:\n" << m << std::endl;
    VectorXd v(2);
    v(0) = 4; v(1) = v(0) - 1;
    std::cout << "Here is the vector v:\n" << v << std::endl;

    //An example 2x2 matrix od doules, initialized differently
    MatrixXd A(2,2);
    A << 1, 2, 2, 3;
    cout << "Here is the matrix A:\n" << A << endl;

    SelfAdjointEigenSolver<MatrixXd> eigensolver(A); //Just an eigensolver object initialized with matrix A, does it solve on construction?
    if (eigensolver.info() != Success) abort(); //An enum, use code navigation to see the definition
    cout << "The eigenvalues of A are:\n" << eigensolver.eigenvalues() << endl; //Eigenvalues are just a vector
    cout << "Here's a matrix whose columns are eigenvectors of A \n"
         << "corresponding to these eigenvalues:\n"
         << eigensolver.eigenvectors() << endl; // eigenvectors are just a matrix

    // Different matrix construction, note 3 instead of X. This denotes a 3x3 matrix
    Matrix3d B;
    Vector3d b; //Vector is just an Xx1 matrix, in this case 3x1. Equivalent to VectorXd b(3);
    B << 1,2,3,  4,5,6,  7,8,10; // Initialization, row by row it seems.
    b << 3, 3, 4;
    cout << "Here is the matrix A:\n" << A << endl;
    cout << "Here is the vector b:\n" << b << endl;

    // Matrix object supports solution a linear problem. colPivHouseholderQr is just one of many ways to handle Ax=b
    Vector3d x = B.colPivHouseholderQr().solve(b); //Here we solve Bx=b, where x is the unknown

    cout << "The solution is:\n" << x << endl;

    return 0;
}

